package com.basic.eyflutter_uikit;

import android.Manifest;
import android.content.Context;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.View;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;

import com.basic.eyflutter_core.utils.MmkvUtils;
import com.basic.eyflutter_core.utils.RedirectUtils;
import com.basic.eyflutter_uikit.beans.ActionItem;
import com.basic.eyflutter_uikit.dialogs.BaseMessageBox;
import com.basic.eyflutter_uikit.enums.DialogButtonsEnum;
import com.basic.eyflutter_uikit.enums.PermissionGrantedType;
import com.basic.eyflutter_uikit.permission.OnGrantedConsumer;
import com.basic.eyflutter_uikit.permission.PermissionManager;
import com.basic.eyflutter_uikit.permission.PermissionState;
import com.cloud.eyutils.HandlerManager;
import com.cloud.eyutils.events.Action1;
import com.cloud.eyutils.events.RunnableParamsN;
import com.cloud.eyutils.launchs.LauncherState;
import com.cloud.eyutils.utils.CollectionUtils;
import com.cloud.eyutils.utils.ConvertUtils;
import com.cloud.eyutils.utils.ObjectJudge;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Author lijinghuan
 * Email:ljh0576123@163.com
 * CreateTime:2019-05-24
 * Description:权限检测工具类
 * Modifier:
 * ModifyContent:
 */
public class PermissionCheckAuxiliary {
    //存储权限组
    private final String STORAGE_PERMISSION = "storage_permission";
    //麦克风权限组
    private final String AUDIO_PERMISSION = "audio_permission";

    /**
     * 检测权限
     *
     * @param activity    activity
     * @param permissions 检测的权限
     * @return true-所有权限均已授权;false-至少有一个权限未授权;
     */
    public boolean check(FragmentActivity activity, String... permissions) {
        HashMap<String, Set<String>> permissionGroup = getPermissionGroup(permissions);
        Map<String, PermissionGrantedType> resultMap = PermissionManager.getInstance().detectionPermissions(activity, permissionGroup);
        for (Map.Entry<String, PermissionGrantedType> entry : resultMap.entrySet()) {
            PermissionGrantedType value = entry.getValue();
            if (value != PermissionGrantedType.granted) {
                return false;
            }
        }
        return true;
    }

    /**
     * 检测权限
     *
     * @param fragmentActivity fragmentActivity
     * @param content          未开启权限时提示内容
     * @param complete         检测完成
     * @param permissions      检测的权限
     */
    public void apply(final FragmentActivity fragmentActivity, final String content, final Action1<PermissionGrantedType> complete, final String... permissions) {
        if (fragmentActivity == null || ObjectJudge.isNullOrEmpty(permissions)) {
            return;
        }
        //权限分组
        try {
            if (ObjectJudge.isMainThread()) {
                HashMap<String, Set<String>> permissionGroup = getPermissionGroup(permissions);
                PermissionManager.getInstance().apply(fragmentActivity, new GrantedConsumerCall(content, fragmentActivity, complete), permissionGroup);
            } else {
                HandlerManager.getInstance().post(new RunnableParamsN<Object>() {
                    @Override
                    public void run(Object... objects) {
                        HashMap<String, Set<String>> permissionGroup = getPermissionGroup(permissions);
                        PermissionManager.getInstance().apply(fragmentActivity, new GrantedConsumerCall(content, fragmentActivity, complete), permissionGroup);
                    }
                });
            }
        } catch (Exception e) {
            //permission apply object error
        }
    }

    /**
     * 检测权限
     *
     * @param fragment    fragment
     * @param content     未开启权限时提示内容
     * @param complete    检测完成
     * @param permissions 检测的权限
     */
    public void apply(final Fragment fragment, final String content, final Action1<PermissionGrantedType> complete, final String... permissions) {
        if (fragment == null || ObjectJudge.isNullOrEmpty(permissions)) {
            return;
        }
        //权限分组
        try {
            if (ObjectJudge.isMainThread()) {
                HashMap<String, Set<String>> permissionGroup = getPermissionGroup(permissions);
                Context context = fragment.getContext();
                PermissionManager.getInstance().apply(fragment, new GrantedConsumerCall(content, context, complete), permissionGroup);
            } else {
                HandlerManager.getInstance().post(new RunnableParamsN<Object>() {
                    @Override
                    public void run(Object... objects) {
                        HashMap<String, Set<String>> permissionGroup = getPermissionGroup(permissions);
                        Context context = fragment.getContext();
                        PermissionManager.getInstance().apply(fragment, new GrantedConsumerCall(content, context, complete), permissionGroup);
                    }
                });
            }
        } catch (Exception e) {
            //permission apply object error
        }
    }

    /**
     * 权限分组
     *
     * @param permissions
     * @return
     */
    private HashMap<String, Set<String>> getPermissionGroup(String... permissions) {
        HashMap<String, Set<String>> permissionMap = new HashMap<>();
        for (String permission : permissions) {
            if (TextUtils.equals(permission, Manifest.permission.READ_EXTERNAL_STORAGE)
                    || TextUtils.equals(permission, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                //存储权限组
                CollectionUtils.put(permissionMap, STORAGE_PERMISSION, permission);
            } else if (TextUtils.equals(permission, Manifest.permission.RECORD_AUDIO)) {
                //麦克风权限组
                CollectionUtils.put(permissionMap, AUDIO_PERMISSION, permission);
            } else {
                CollectionUtils.put(permissionMap, permission, permission);
            }
        }
        return permissionMap;
    }

    /**
     * 检测权限
     *
     * @param fragmentActivity fragmentActivity
     * @param permissions      permissions
     * @return permission-state
     */
    public Map<String, PermissionGrantedType> detection(FragmentActivity fragmentActivity, String... permissions) {
        if (fragmentActivity == null || ObjectJudge.isNullOrEmpty(permissions)) {
            return new HashMap<>();
        }
        //权限分组
        HashMap<String, Set<String>> permissionGroup = getPermissionGroup(permissions);
        return PermissionManager.getInstance().detectionPermissions(fragmentActivity, permissionGroup);
    }

    private class GrantedConsumerCall extends OnGrantedConsumer {

        private String content;
        private WeakReference<Context> contextRef;

        public GrantedConsumerCall(String content, Context context, Object... extras) {
            super(extras);
            this.content = content;
            this.contextRef = new WeakReference<>(context);
        }

        @Override
        public void acceptEach(Map<String, PermissionState> authStates, boolean isApplySuccess) {
            if (isApplySuccess || null != authStates) {
                //权限成功回调
                for (Map.Entry<String, PermissionState> entry : authStates.entrySet()) {
                    //缓存权限状态
                    Context applicationContext = LauncherState.getApplicationContext();
                    MmkvUtils.getInstance(applicationContext).putString("permission_cache_withid", entry.getKey(), "string", entry.getValue());
                }
            }
        }

        @Override
        public void accept(PermissionGrantedType grantedType, Object[] extras) {
            if (ObjectJudge.isNullOrEmpty(extras)) {
                return;
            }
            if (contextRef == null) {
                return;
            }
            Context context = contextRef.get();
            if (context == null) {
                return;
            }
            Action1<PermissionGrantedType> complete = (Action1<PermissionGrantedType>) extras[0];
            if (complete != null) {
                complete.call(grantedType);
            }

            if (TextUtils.isEmpty(content)) {
                //提示内容为空则不弹出自定义提示框
                return;
            }
            if (grantedType == PermissionGrantedType.askNeverAgain) {
                if (TextUtils.isEmpty(content)) {
                    Context applicationContext = LauncherState.getApplicationContext();
                    if (applicationContext != null) {
                        RedirectUtils.startAppSettings(applicationContext);
                    }
                } else {
                    messageBox.setShowClose(false);
                    messageBox.setTitle("权限校验");
                    messageBox.setContent(content);
                    messageBox.setContentGravity(Gravity.CENTER);
                    messageBox.setTarget("permission_check_target", complete);
                    ActionItem[] actionItems = {
                            new ActionItem("cancel", "取消"),
                            new ActionItem("go_setting", "去设置")
                    };
                    messageBox.setButtons(actionItems);
                    messageBox.show(context, DialogButtonsEnum.Custom);
                }
            } else if (grantedType == PermissionGrantedType.needStrongReminder) {
                messageBox.setShowClose(false);
                messageBox.setTitle("权限校验");
                messageBox.setContent(String.format("由于您的操作系统版本低于6.0且勾选不再提醒，使用本功能需手动在手机系统设置中开启%s权限，如再次请求或在设置中无法修改，请先将卸载应用，然后重新安装应用。", getSRPItems()));
                messageBox.setContentGravity(Gravity.LEFT);
                messageBox.setTarget("permission_check_strong_reminder_target", complete);
                ActionItem[] actionItems = {
                        new ActionItem("know", "我知道了"),
                        new ActionItem("go_setting", "去设置")
                };
                messageBox.setButtons(actionItems);
                messageBox.show(context, DialogButtonsEnum.Custom);
            }
        }
    }

    private BaseMessageBox messageBox = new BaseMessageBox() {
        @Override
        public boolean onItemClickListener(View v, String cmdid, String target, Object extraData) {
            if (TextUtils.equals(target, "permission_check_target") ||
                    TextUtils.equals(target, "permission_check_strong_reminder_target")) {
                if (TextUtils.equals(cmdid, "go_setting")) {
                    Context applicationContext = LauncherState.getApplicationContext();
                    if (applicationContext != null) {
                        RedirectUtils.startAppSettings(applicationContext);
                    }
                }
            }
            return true;
        }
    };

    /**
     * 获取申请的权限
     *
     * @param permissionTags https://wiki.ttsing.com/pages/viewpage.action?pageId=30081266
     * @return Manifest.permission.[...]对应的权限
     */
    public String[] getPermissions(String... permissionTags) {
        if (ObjectJudge.isNullOrEmpty(permissionTags)) {
            return new String[0];
        }
        Set<String> permissionSet = new HashSet<>();
        for (String permissionTag : permissionTags) {
            if (TextUtils.isEmpty(permissionTag)) {
                continue;
            }
            if (TextUtils.equals(permissionTag, "RECORD_AUDIO")) {
                permissionSet.add(Manifest.permission.RECORD_AUDIO);
            } else if (TextUtils.equals(permissionTag, "PHONE_STATE")) {
                permissionSet.add(Manifest.permission.READ_PHONE_STATE);
            } else if (TextUtils.equals(permissionTag, "EXTERNAL_STORAGE")) {
                permissionSet.add(Manifest.permission.READ_EXTERNAL_STORAGE);
                permissionSet.add(Manifest.permission.WRITE_EXTERNAL_STORAGE);
            }
        }
        return ConvertUtils.toArray(permissionSet);
    }
}
