import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_core/kit/utils/set/list_extention.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';

class ButtonArgs {
  /// 按钮文本
  final String text;

  /// 按钮文本颜色
  final Color? color;

  /// 是否加粗
  final bool isBold;

  /// 唯一标识符
  final String action;

  @deprecated
  final bool? isHighlighted;

  ButtonArgs({this.text = '', this.color, this.isBold = false, this.action = '', this.isHighlighted});
}

class DialogEntry {
  YYDialog? dialog;
}

class ListItem {
  String text;

  String value;

  bool? check;

  ListItem({required this.text, required this.value, this.check});
}

typedef OnButtonPressed(ButtonArgs buttonArgs);
typedef OnListItemPressed(List<ListItem> selectItems);
typedef OnDialogBuildCall(YYDialog dialog);

class Dialogs {
  factory Dialogs() => _getInstance();

  static Dialogs get instance => _getInstance();
  static Dialogs? _instance;

  static Dialogs _getInstance() {
    return _instance ??= new Dialogs._internal();
  }

  Dialogs._internal();

  /// 显示dialog(message优先级大于contentView)
  /// [context] widget context
  /// [backgroundColor] dialog背景颜色
  /// [borderRadius] 边框圆角大小
  /// [title] 标题
  /// [textAlign] 标题
  /// [message] 消息内容与contentView互斥,若设置message后contentView无效
  /// [contentView] 内容自定义视图与message互斥
  /// [buttonArgs] 按钮列表
  /// [buttonPressed] 按钮事件回调
  /// [isClickOutsideDismiss] 点击dialog外部是否消失
  /// [margin] dialog外边距
  /// [buildCall] 构建完成回调
  YYDialog showDialog(BuildContext context,
      {Color backgroundColor = Colors.white,
      BorderRadius borderRadius = const BorderRadius.all(Radius.circular(6)),
      String title = '',
      String message = '',
      Widget? contentView,
      List<ButtonArgs> buttonArgs = const <ButtonArgs>[],
      OnButtonPressed? buttonPressed,
      bool isClickOutsideDismiss = false,
      OnDialogBuildCall? buildCall,
      EdgeInsetsGeometry? padding,
      Gravity? gravity,
      bool isShowClose = false,
      TextAlign? textAlign,
      margin = const EdgeInsets.only(left: 20, right: 20),
      OnDialogDismissCall? dismissCall}) {
    var entry = DialogEntry();
    var dialogPanel = _MessageDialogPanel(
      backgroundColor: backgroundColor,
      title: title,
      message: message,
      contentView: contentView,
      buttonArgs: buttonArgs,
      buttonPressed: buttonPressed,
      dialogEntry: entry,
      messageAlign: textAlign,
      padding: padding ?? EdgeInsets.only(left: 20, top: title.isEmptyString ? 25 : 16, right: 20, bottom: 25),
      isShowClose: isShowClose,
    );
    var dialog = DialogPanel.showMessageDialog(context,
        container: dialogPanel,
        borderRadius: borderRadius,
        barrierDismissible: isClickOutsideDismiss,
        dismissCall: dismissCall,
        gravity: gravity,
        margin: margin);
    entry.dialog = dialog;
    if (buildCall != null) {
      buildCall(dialog);
    }
    return dialog;
  }

  /// 显示列表样式dialog
  /// [context] widget context
  /// [backgroundColor] dialog背景颜色
  /// [borderRadius] 边框圆角大小
  /// [isClickOutsideDismiss] 点击dialog外部是否消失
  /// [itemPressed] 列表元素单击监听
  /// [dx] x方向上的偏移量
  /// [dy] y方向上的偏移量
  /// [buildCall] 构建完成回调
  /// [title] 标题
  /// [submit] 提交按钮
  /// [submitText] 提交按钮文本
  /// [isPreSelect] true-选择后需要由确认完成;false-选择直接完成;
  YYDialog showListDialog(
    BuildContext context, {
    Color backgroundColor = Colors.white,
    BorderRadius borderRadius = const BorderRadius.only(topLeft: Radius.circular(4), topRight: Radius.circular(4)),
    bool isClickOutsideDismiss = false,
    List<ListItem> items = const <ListItem>[],
    OnListItemPressed? itemPressed,
    OnDialogBuildCall? buildCall,
    Gravity? gravity,
    double dx = 0,
    double dy = 0,
    OnDialogDismissCall? dismissCall,
    String? title,
    Color? titleColor,
    double? titleSize,
    Widget? submit,
    String? submitText,
    Color? submitTextColor,
    double? submitTextSize,
    BorderRadiusGeometry submitBorderRadius = const BorderRadius.all(Radius.circular(5)),
    Color? submitBackgroundColor,
    bool isPreSelect = true,
    bool isMultiSelect = false,
    String leastSelectHintText = "",
    Color? listSplitLineColor,
    double itemHeight = 42,
    Color? itemTextColor,
    double? itemTextSize,
    Widget? multiSelectIcon,
    Widget? multiUnSelectIcon,
    Widget? singleSelectIcon,
    Widget? singleUnSelectIcon,
  }) {
    var entry = DialogEntry();
    var dialogPanel = _ListDialogPanel(
      backgroundColor: backgroundColor,
      items: items,
      itemPressed: itemPressed,
      dialogEntry: entry,
      title: title,
      titleColor: titleColor,
      titleSize: titleSize,
      submit: submit,
      submitText: submitText,
      submitTextColor: submitTextColor,
      submitTextSize: submitTextSize,
      submitBorderRadius: submitBorderRadius,
      submitBackgroundColor: submitBackgroundColor,
      isPreSelect: isPreSelect,
      isMultiSelect: isMultiSelect,
      listSplitLineColor: listSplitLineColor,
      leastSelectHintText: leastSelectHintText,
      model: _ListDialogModel(),
      itemHeight: itemHeight,
      itemTextColor: itemTextColor,
      itemTextSize: itemTextSize,
      multiSelectIcon: multiSelectIcon,
      multiUnSelectIcon: multiUnSelectIcon,
      singleSelectIcon: singleSelectIcon,
      singleUnSelectIcon: singleUnSelectIcon,
    );
    var dialog = DialogPanel.showMessageDialog(
      context,
      container: dialogPanel,
      borderRadius: borderRadius,
      barrierDismissible: isClickOutsideDismiss,
      dismissCall: dismissCall,
      gravity: gravity,
      dx: dx,
      dy: dy,
    );
    entry.dialog = dialog;
    if (buildCall != null) {
      buildCall(dialog);
    }
    return dialog;
  }

  /// 显示顶部浮动组件
  /// [context] widget context
  /// [contentView] 组件视图
  /// [margin] 外边距
  FloatingPanel showTopFloatingWidget(
      {required BuildContext context,
      required Widget contentView,
      EdgeInsetsGeometry? margin,
      Color? backgroundColor}) {
    var floatingPanel = FloatingPanel();
    floatingPanel.show(context, contentView, margin: margin, backgroundColor: backgroundColor);
    return floatingPanel;
  }
}

class _ListDialogModel extends CusChangeNotifier {}

class _ListDialogPanel extends StatelessWidget {
  /// 背景颜色(默认白色)
  final Color? backgroundColor;

  final List<ListItem>? items;

  final OnListItemPressed? itemPressed;

  final DialogEntry? dialogEntry;

  final String? title;

  final Color? titleColor;

  final double? titleSize;

  final Widget? submit;

  final String? submitText;

  final Color? submitTextColor;

  final double? submitTextSize;

  final BorderRadiusGeometry submitBorderRadius;

  final Color? submitBackgroundColor;

  final bool isPreSelect;

  final bool isMultiSelect;

  final _ListDialogModel model;

  final String leastSelectHintText;

  final Color? listSplitLineColor;

  final double itemHeight;

  final Color? itemTextColor;

  final double? itemTextSize;

  final Widget? multiSelectIcon;

  final Widget? multiUnSelectIcon;

  final Widget? singleSelectIcon;

  final Widget? singleUnSelectIcon;

  const _ListDialogPanel({
    Key? key,
    this.backgroundColor,
    this.items,
    this.itemPressed,
    this.dialogEntry,
    this.title,
    this.titleColor,
    this.titleSize,
    this.submit,
    this.submitText,
    this.submitTextColor,
    this.submitTextSize,
    this.submitBorderRadius = const BorderRadius.all(Radius.circular(5)),
    this.submitBackgroundColor,
    this.isPreSelect = true,
    this.isMultiSelect = false,
    required this.model,
    this.leastSelectHintText = "",
    this.listSplitLineColor,
    this.itemHeight = 42,
    this.itemTextColor,
    this.itemTextSize,
    this.multiSelectIcon,
    this.multiUnSelectIcon,
    this.singleSelectIcon,
    this.singleUnSelectIcon,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StateUtils.instance.subscribeNotifierWidget<_ListDialogModel>(model, (context, model) {
      return Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Container(
            height: 44,
            color: backgroundColor,
            child: Row(
              mainAxisSize: MainAxisSize.min,
              children: [
                Expanded(child: buildTitle()),
                isPreSelect ? (submit ?? buildConfirm()) : Container(),
              ],
            ),
          ),
          buildList(context),
        ],
      );
    });
  }

  Widget buildTitle() {
    return Padding(
      padding: EdgeInsets.only(left: 12, right: 8),
      child: Text(
        title ?? "",
        style: TextStyle(
          color: titleColor,
          fontSize: titleSize,
          overflow: TextOverflow.ellipsis,
        ),
        maxLines: 1,
        textAlign: isPreSelect ? TextAlign.start : TextAlign.center,
      ),
    );
  }

  Widget buildConfirm() {
    return Padding(
      padding: EdgeInsets.only(right: 12),
      child: TextButton(
        child: Text(
          submitText ?? "",
          style: TextStyle(color: submitTextColor, fontSize: submitTextSize),
        ),
        style: ButtonStyle(
          padding: MaterialStateProperty.all(EdgeInsets.zero),
          shape: MaterialStateProperty.all(RoundedRectangleBorder(borderRadius: submitBorderRadius)),
          backgroundColor: MaterialStateProperty.all(submitBackgroundColor),
          visualDensity: VisualDensity.compact,
          minimumSize: MaterialStateProperty.all(Size(56, 36)),
        ),
        onPressed: completeSelect,
      ),
    );
  }

  void completeSelect() {
    var checkItems = selectItems;
    if (checkItems.isEmptyList) {
      ToastUtils.instance.show(leastSelectHintText);
      return;
    }
    if (itemPressed != null) {
      itemPressed!(checkItems);
    }
    dialogEntry?.dialog?.dismiss();
  }

  Widget buildList(BuildContext context) {
    return ScrollVerticalView(
      child: Container(
        color: backgroundColor ?? Colors.white,
        constraints: BoxConstraints(
          maxHeight: MediaQuery.of(context).size.height * 2 / 3,
          minHeight: 120,
        ),
        child: ListView.separated(
          padding: EdgeInsets.zero,
          physics: NeverScrollableScrollPhysics(),
          separatorBuilder: (BuildContext context, int index) {
            return Container(
              color: listSplitLineColor,
              margin: EdgeInsets.only(left: 10, right: 10),
              height: 0.8,
            );
          },
          itemBuilder: (BuildContext context, int index) {
            var item = items![index];
            return buildItemView(context, item);
          },
          shrinkWrap: true,
          itemCount: items?.length ?? 0,
        ),
      ),
    );
  }

  Widget buildItemView(BuildContext context, ListItem item) {
    return TextButton(
      child: Container(
          height: itemHeight,
          alignment: Alignment.centerLeft,
          padding: EdgeInsets.only(left: 10, right: 10),
          child: Row(
            mainAxisSize: MainAxisSize.min,
            children: [
              Expanded(
                child: Text(
                  item.text,
                  style: TextStyle(fontSize: itemTextSize, color: itemTextColor),
                  maxLines: 1,
                  overflow: TextOverflow.ellipsis,
                  softWrap: true,
                ),
              ),
              buildIcon(item),
            ],
          )),
      style: ButtonStyle(
        padding: MaterialStateProperty.all(EdgeInsets.zero),
        shape: MaterialStateProperty.all(RoundedRectangleBorder(borderRadius: BorderRadius.zero)),
        backgroundColor: MaterialStateProperty.all(Colors.transparent),
        visualDensity: VisualDensity.compact,
        minimumSize: MaterialStateProperty.all(Size.zero),
      ),
      onPressed: () {
        if (isPreSelect) {
          checkSelectItems(item);
        }
        if (!isPreSelect) {
          if (itemPressed != null) {
            itemPressed!(selectItems);
          }
          dialogEntry?.dialog?.dismiss();
        }
      },
    );
  }

  Widget buildItemIcon(String imageName) {
    return Image.asset(
      "assets/images/$imageName.png",
      package: "eyflutter_uikit",
      width: 20,
      height: 20,
    );
  }

  Widget buildIcon(ListItem item) {
    if (isPreSelect) {
      Widget icon;
      if (isMultiSelect) {
        if (item.check ?? false) {
          icon = multiSelectIcon ?? buildItemIcon("ey_m_check_icon");
        } else {
          icon = multiUnSelectIcon ?? buildItemIcon("ey_m_uncheck_icon");
        }
      } else {
        if (item.check ?? false) {
          icon = singleSelectIcon ?? buildItemIcon("ey_check_icon");
        } else {
          icon = singleUnSelectIcon ?? buildItemIcon("ey_uncheck_icon");
        }
      }
      return Padding(
        padding: EdgeInsets.only(left: 8),
        child: icon,
      );
    } else {
      return Container();
    }
  }

  List<ListItem> get selectItems {
    List<ListItem> lst = [];
    var count = items?.length ?? 0;
    for (int i = 0; i < count; i++) {
      var element = items![i];
      if (element.check ?? false) {
        lst.add(element);
      }
    }
    return lst;
  }

  void checkSelectItems(ListItem currentItem) {
    var count = items?.length ?? 0;
    for (int i = 0; i < count; i++) {
      var element = items![i];
      if (isMultiSelect) {
        if (element.value == currentItem.value) {
          element.check = !(currentItem.check ?? false);
          break;
        }
      } else {
        if (element.value == currentItem.value) {
          element.check = true;
        } else {
          element.check = false;
        }
      }
    }
    model.notifierLazy();
  }
}

class _MessageDialogPanel extends StatelessWidget {
  /// 背景颜色(默认白色)
  final Color backgroundColor;

  /// 标题
  final String? title;

  /// 消息
  final String? message;

  /// 自定义视图
  final Widget? contentView;

  /// 内容对齐(有标题默认居中，无标题默认居左)
  final TextAlign? messageAlign;

  /// 按钮标识(标识顺序为按钮从左到右顺序)
  final List<ButtonArgs> buttonArgs;

  /// 操作项回调事件
  final OnButtonPressed? buttonPressed;

  final DialogEntry? dialogEntry;

  /// 内边距
  final EdgeInsetsGeometry? padding;

  /// 是否显示关闭按钮
  final bool isShowClose;

  _MessageDialogPanel(
      {this.backgroundColor = Colors.white,
      this.title,
      this.message,
      this.contentView,
      this.buttonArgs = const <ButtonArgs>[],
      this.messageAlign,
      this.buttonPressed,
      this.dialogEntry,
      this.padding,
      this.isShowClose = false})
      : super();

  @override
  Widget build(BuildContext context) {
    return Container(
      color: backgroundColor,
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: combColumn(),
      ),
    );
  }

  Widget buildTitle() {
    return Container(
      height: 42,
      color: Colors.transparent,
      alignment: Alignment.bottomCenter,
      padding: EdgeInsets.only(left: 20, top: 10, right: 20),
      child: Stack(
        children: [
          Text(
            title ?? "",
            style: TextStyle(fontSize: 16, color: Color(0xff222222), letterSpacing: 0.82, fontWeight: FontWeight.bold),
            textAlign: TextAlign.center,
            overflow: TextOverflow.ellipsis,
            maxLines: 1,
          ),
          (isShowClose)
              ? GestureDetector(
                  child: Padding(
                    padding: EdgeInsets.fromLTRB(30, 2, 30, 2),
                    child: Icon(
                      Icons.close,
                      color: Color(0xffC8CACC),
                      size: 22,
                    ),
                  ),
                  onTap: () {
                    dialogEntry?.dialog?.dismiss();
                  },
                )
              : Stack(),
        ],
      ),
    );
  }

  Widget buildContent() {
    return Container(
      width: double.infinity,
      padding: padding ?? EdgeInsets.only(left: 20, top: title.isEmptyString ? 25 : 16, right: 20, bottom: 25),
      color: Colors.transparent,
      child: message.isEmptyString
          ? (contentView ?? Container())
          : Text(
              message ?? "",
              style: TextStyle(
                  fontSize: 16,
                  color: title.isEmptyString ? Color(0xff222222) : Color(0xff666666),
                  letterSpacing: 0.77),
              maxLines: 10,
              overflow: TextOverflow.ellipsis,
              textAlign: contentAlign(),
            ),
    );
  }

  TextAlign? contentAlign() {
    if (title.isEmptyString) {
      return messageAlign == null ? TextAlign.left : messageAlign;
    } else {
      return messageAlign == null ? TextAlign.center : messageAlign;
    }
  }

  Widget buildDivider() {
    return Divider(
      height: 1,
      indent: 0.0,
      color: Color(0xffefefef),
    );
  }

  Widget buildVerDivider() {
    return VerticalDivider(
      indent: 0.0,
      width: 1,
      color: Color(0xffefefef),
    );
  }

  Widget buildButtonsView() {
    return Container(
      height: 45,
      color: Colors.transparent,
      padding: EdgeInsets.all(0.0),
      child: Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: combButton(),
      ),
    );
  }

  Widget buildButton(ButtonArgs buttonArgs) {
    return Expanded(
      flex: 1,
      child: TextButton(
        onPressed: () {
          dialogEntry?.dialog?.dismiss();
          if (buttonPressed != null) {
            buttonPressed!(buttonArgs);
          }
        },
        child: Container(
          height: 45,
          alignment: Alignment.center,
          decoration: BoxDecoration(borderRadius: BorderRadius.zero),
          child: Text(
            buttonArgs.text,
            style: TextStyle(
                fontSize: 16,
                fontWeight: buttonArgs.isBold ? FontWeight.bold : FontWeight.normal,
                letterSpacing: 0.77,
                color: buttonArgs.color ?? Color(0xFF999999)),
            textAlign: TextAlign.center,
          ),
        ),
      ),
    );
  }

  List<Widget> combColumn() {
    List<Widget> lst = [];
    if (title.isNotEmptyString) {
      lst.add(buildTitle());
    }
    lst.add(buildContent());
    if (buttonArgs.isNotEmptyList) {
      lst.add(buildDivider());
      lst.add(buildButtonsView());
    }
    return lst;
  }

  List<Widget> combButton() {
    List<Widget> lst = [];
    var len = buttonArgs.length;
    for (var i = 0; i < len; i++) {
      var element = buttonArgs[i];
      if (i > 0) {
        lst.add(buildVerDivider());
      }
      lst.add(buildButton(element));
    }
    return lst;
  }
}
