import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:flutter/material.dart';

typedef OnFlowItemClick = void Function(int position);

typedef OnFlowItemChangeCall = void Function(int position, bool isCheck, dynamic data);

typedef OnFlowItemBuildCall = void Function(int position, OnFlowItemController controller);

mixin OnFlowItemController {
  /// 通知更新显示文本
  void notifyText(String text);

  /// 通知状态是否被选中
  void notifyCheck(bool check);

  /// 通知是否被禁用
  void notifyEnable(bool enable);
}

class FlowItemWidget extends StatefulWidget {
  /// 标签文本
  final String text;

  /// 文本颜色
  final Color? textColor;

  /// 选中文本颜色
  final Color? selectTextColor;

  /// 文本大小
  final double? textSize;

  /// 选项单击事件
  final OnFlowItemClick? itemClick;

  /// 组件高度
  final double? height;

  /// 该项是否被选中
  final bool isCheck;

  /// 该项是否启用
  final bool enable;

  /// 选项改变监听
  final OnFlowItemChangeCall? changeCall;

  /// 数据项
  final dynamic data;

  /// item index
  final int position;

  /// 构建完成回调
  final OnFlowItemBuildCall? buildCall;

  /// 背景颜色
  final Color backgroundColor;

  /// 边框颜色
  final Color borderColor;

  /// 选择项背景颜色
  final Color selectedBackgroundColor;

  /// 选择项边框颜色
  final Color selectedBorderColor;

  /// 禁用背景颜色
  final Color disableBackgroundColor;

  /// 内边距
  final EdgeInsetsGeometry padding;

  /// 标记组件
  final Widget? tagWidget;

  /// 边框圆角
  final BorderRadiusGeometry borderRadius;

  const FlowItemWidget({
    Key? key,
    required this.text,
    this.textColor,
    this.selectTextColor,
    this.textSize,
    this.itemClick,
    this.height,
    this.isCheck = false,
    this.enable = true,
    this.changeCall,
    this.data,
    this.position = 0,
    this.buildCall,
    this.backgroundColor = Colors.white,
    this.borderColor = const Color(0xffefefef),
    this.selectedBackgroundColor = Colors.white,
    this.selectedBorderColor = const Color(0xffFF2E18),
    this.disableBackgroundColor = const Color(0xfff8f8f8),
    this.padding = const EdgeInsets.only(left: 4, right: 4),
    this.tagWidget,
    this.borderRadius = const BorderRadius.all(Radius.circular(3)),
  }) : super(key: key);

  @override
  _FlowItemWidgetState createState() => _FlowItemWidgetState();
}

class _FlowItemWidgetState extends State<FlowItemWidget> with OnFlowItemController {
  String _text = "";
  bool _check = false;
  bool _enable = true;

  @override
  void initState() {
    _text = widget.text;
    _check = widget.isCheck;
    _enable = widget.enable;
    super.initState();
    if (widget.buildCall != null) {
      widget.buildCall!(widget.position, this);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: widget.height,
      child: TextButton(
        child: _buildContent(),
        style: ButtonStyle(
          textStyle: MaterialStateProperty.all(TextStyle(
            color: (_check && _enable) ? widget.selectTextColor : widget.textColor,
            fontSize: widget.textSize,
          )),
          padding: MaterialStateProperty.all(widget.padding),
          shape: MaterialStateProperty.all(RoundedRectangleBorder(borderRadius: widget.borderRadius)),
        ),
        onPressed: (_enable)
            ? () {
                if (widget.changeCall != null) {
                  widget.changeCall!(widget.position, !(_check), widget.data);
                }
                if (widget.itemClick != null) {
                  widget.itemClick!(widget.position);
                }
              }
            : null,
      ),
      decoration: BoxDecoration(
        color: _background(),
        border: _border(),
        borderRadius: widget.borderRadius,
      ),
    );
  }

  Widget _buildContent() {
    if (widget.tagWidget == null) {
      return Text(_text,
          style: TextStyle(
            color: (_check && _enable) ? widget.selectTextColor : widget.textColor,
            fontSize: widget.textSize,
          ));
    }
    return Container(
      child: Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          widget.tagWidget!,
          Padding(
            padding: EdgeInsets.only(left: 8),
            child: Text(_text,
                style: TextStyle(
                  color: (_check && _enable) ? widget.selectTextColor : widget.textColor,
                  fontSize: widget.textSize,
                )),
          )
        ],
      ),
    );
  }

  Border _border() {
    if (_check && _enable) {
      return Border.all(width: 0.8, color: widget.selectedBorderColor);
    } else {
      return Border.all(width: 0.8, color: widget.borderColor);
    }
  }

  Color _background() {
    if (_enable) {
      if (_check) {
        return widget.selectedBackgroundColor;
      } else {
        return widget.backgroundColor;
      }
    } else {
      return widget.disableBackgroundColor;
    }
  }

  @override
  void notifyText(String text) {
    if (text.isNotEmptyString) {
      setState(() {
        _text = text;
      });
    }
  }

  @override
  void notifyCheck(bool check) {
    setState(() {
      _check = check;
    });
  }

  @override
  void notifyEnable(bool enable) {
    setState(() {
      _enable = enable;
    });
  }
}
