import 'dart:io';
import 'dart:ui';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/enums/screen_orientation.dart';
import 'package:eyflutter_uikit/handler/log_handler.dart';
import 'package:eyflutter_uikit/utils/media_utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

/// 初始化同步完成回调
typedef OnInitSyncCall = void Function();

/// flutter engine initialize
mixin OnInitializeListener {
  /// 构建应用启动组件
  Widget buildAppWidget({String defaultRout = "/"});

  /// 应用启动回调(组件创建之前执行)
  Future<void> startSyncCall({required OnInitSyncCall syncCall});

  /// 应用启动回调(组件创建后执行)
  void startCall();

  /// 获取应用环境
  int environment();

  /// crash 上报回调
  void crashReportCall(CrashInfo crashInfo) {}

  /// crash 执行完成回调
  void crashCompletedCall();
}

class FlutterInitialize {
  factory FlutterInitialize() => _getInstance();

  static FlutterInitialize get instance => _getInstance();
  static FlutterInitialize? _instance;

  FlutterInitialize._internal();

  static FlutterInitialize _getInstance() {
    return _instance ??= FlutterInitialize._internal();
  }

  /// flutter engine 启动初始化
  void start({required OnInitializeListener listener}) async {
    WidgetsFlutterBinding.ensureInitialized();
    Logger.instance.builder?.setLogListener(LogHandler()).setPrintLog(true);
    EyFlutter.instance.setStartTime(time: System.currentTimeMillis);
    listener.startSyncCall(syncCall: () {
      var appWidget = listener.buildAppWidget(defaultRout: window.defaultRouteName);
      CrashHandler.instance.build(appWidget,
          report: (crashInfo) {
            listener.crashReportCall(crashInfo);
          },
          environment: listener.environment(),
          completedCall: () {
            listener.crashCompletedCall();
          });
      listener.startCall();
      MediaUtils.instance.initialize(375);
      LangStorage.instance.loadLang();
      overlayStyle();
    });
  }

  void overlayStyle() {
    if (Platform.isAndroid) {
      SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle(
        statusBarColor: Colors.transparent,
        statusBarIconBrightness: Brightness.dark,
        statusBarBrightness: Brightness.light,
      ));
    }
    MediaUtils.instance.screen(ScreenOrientation.portrait);
  }
}
