import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/utils/media_utils.dart';

/// 腾讯云图片url地址处理
/// https://cloud.tencent.com/document/product/460/36540

/// 阿里云图片url地址处理
/// https://help.aliyun.com/document_detail/144582.html

//图片裁剪对齐枚举
//  -------------------------------------
//  | northwest |  north  |  northeast  |
//  -------------------------------------
//  | west      |  center |  east       |
//  -------------------------------------
//  | southwest |  south  |  southeast  |
//  -------------------------------------
// 当 gravity 参数设置为 center 时，dx、dy 参数无效。
// 当 gravity 参数设置为 north 或 south 时，dx 参数无效（水印会水平居中）。
// 当 gravity 参数设置为 west 或 east 时，dy 参数无效（水印会垂直居中）。
enum ImageCutGravity { northwest, north, northeast, west, center, east, southwest, south, southeast }

//图片裁剪对齐枚举
//  -------------------------------------
//  | nw           |  north  |  ne      |
//  -------------------------------------
//  | west         |  center |  east    |
//  -------------------------------------
//  | sw           |  south  |  se      |
//  -------------------------------------
enum ImageCut { nw, north, ne, west, center, east, sw, south, se }

//图片裁剪方式
enum ImageCutType {
  //根据width、height常规裁剪
  normal,
  //内嵌圆裁剪
  iradius,
  //圆角裁剪
  rradius,
  //智能裁剪(基于图片中的人脸位置进行缩放裁剪,如果图片中没有识别到人脸，则返回原图)
  scrop
}

enum ZoomType {
  /// 无
  none,

  /// 等比缩放
  geometric,
}

class ImageUrlProcess {
  factory ImageUrlProcess() => _getInstance();

  static ImageUrlProcess get instance => _getInstance();
  static ImageUrlProcess? _instance;

  ImageUrlProcess._internal();

  static ImageUrlProcess _getInstance() {
    return _instance ??= new ImageUrlProcess._internal();
  }

  //1-正常地址；2-空、非url规则、已经带有后缀、三方图片地址;3-需要去除参数;4-gif;
  int _checkUrl(String url) {
    if (url.isEmptyString || !url.isUrl) {
      return 2;
    }
    var uri = Uri.parse(url);
    var segments = uri.pathSegments;
    if (segments.isEmptyList) {
      return 2;
    }
    var last = segments.last;
    if (last.startsWith("0x") && last.length == 6) {
      return 2;
    }
    var paramsTagIndex = url.indexOf("?");
    if (paramsTagIndex >= 0) {
      var params = url.substring(paramsTagIndex);
      if (params.length > 1) {
        return 2;
      } else {
        return 3;
      }
    }
    return 1;
  }

  String _cutZoomAliyunAssembly(
      {required String url,
      required double width,
      required double height,
      required ImageCut cut,
      required double radius,
      required ZoomType zoomType}) {
    var urlBuffer = StringBuffer(url);
    urlBuffer..write("?x-oss-process=image/quality,q_80");
    //图片自适应旋转
    urlBuffer..write("/auto-orient,1");
    //jpg图片格式
    urlBuffer..write("/format,webp");
    //输出为渐进式 jpg 格式。Mode 可为0或1。0：表示不开启渐进式；1：表示开启渐进式。
    urlBuffer..write("/interlace,1");
    //锐化(使图片相对原图较清晰)
    urlBuffer..write("/sharpen,100");
    if (zoomType == ZoomType.geometric) {
      urlBuffer..write("/resize,w_${width.toNumInt},m_lfit");
      if (radius > 0) {
        if (radius * 2 >= width) {
          urlBuffer..write("/circle,r_${radius.toNumInt}");
        } else {
          urlBuffer..write("/rounded-corners,r_${radius.toNumInt}");
        }
      }
    } else {
      if (url.suffixName != "png") {
        urlBuffer..write("/crop,w_${width.toNumInt},h_${height.toNumInt},g_${cut.name}");
        if (radius > 0) {
          if (radius * 2 >= width) {
            urlBuffer..write("/circle,r_${radius.toNumInt}");
          } else {
            urlBuffer..write("/rounded-corners,r_${radius.toNumInt}");
          }
        }
      }
    }
    return urlBuffer.toString();
  }

  String _cutZoomAssembly(
      {required String url,
      required double width,
      required double height,
      required ImageCutGravity cutGravity,
      required ImageCutType cutType,
      required double radius,
      required ZoomType zoomType}) {
    var urlBuffer = StringBuffer(url);
    urlBuffer..write("?imageMogr2");
    if (zoomType == ZoomType.geometric) {
      urlBuffer
        ..write("/thumbnail/")
        ..write(width.toNumInt)
        ..write("x");
    } else {
      if (cutType == ImageCutType.normal) {
        urlBuffer
          ..write("/crop/")
          ..write(width.toNumInt)
          ..write("x")
          ..write(height.toNumInt)
          ..write("/gravity/")
          ..write(cutGravity.name);
      } else if (cutType == ImageCutType.scrop) {
        urlBuffer
          ..write("/scrop/")
          ..write(width.toNumInt)
          ..write("x")
          ..write(height.toNumInt);
      } else {
        if (url.suffixName != "png") {
          if (cutType == ImageCutType.iradius) {
            urlBuffer
              ..write("/iradius/")
              ..write(radius.toNumInt);
          } else if (cutType == ImageCutType.rradius) {
            urlBuffer
              ..write("/rradius/")
              ..write(radius.toNumInt);
          }
        }
      }
    }
    //图片自适应旋转
    urlBuffer..write("/auto-orient");
    //jpg图片格式
    urlBuffer..write("/format/webp");
    //输出为渐进式 jpg 格式。Mode 可为0或1。0：表示不开启渐进式；1：表示开启渐进式。
    urlBuffer..write("/interlace/1");
    //图片质量
    urlBuffer..write("/quality/80!");
    //锐化(使图片相对原图较清晰)
    urlBuffer..write("/sharpen/70");
    //去除图片原信息
    urlBuffer..write("/strip");
    return urlBuffer.toString();
  }

  String _imageUrl(String url) {
    var type = _checkUrl(url);
    if (type == 2) {
      return "";
    } else if (type == 3) {
      var index = url.indexOf("?");
      url = url.substring(0, index);
    }
    if (url.suffixName == "gif") {
      return url;
    }
    return url;
  }

  String _aliyunZoomHandle(String url, double width, double height, ImageCut cut, double radius, ZoomType zoomType) {
    var pixelRatio = MediaUtils.instance.pixelRatio;
    if (pixelRatio < 1) {
      pixelRatio = 1;
    } else {
      pixelRatio = pixelRatio * 1.2;
    }
    return _cutZoomAliyunAssembly(
        url: url, width: width * pixelRatio, height: height * pixelRatio, cut: cut, radius: radius, zoomType: zoomType);
  }

  /// 配置图片url
  /// png图片时圆角裁剪无效
  /// [url] 原图片地址
  /// [width] 图片宽度(必填项)
  /// [height] 图片高度(裁剪时需要)
  /// [cutGravity] 图片裁剪对齐参考点(isPortrait==true时该参数无效),默认以center为参考点;
  /// [radius] 裁剪半径、圆角半径;(默认为2，方便图片圆角半径参数设置)
  /// [zoomType] 缩放类型与裁剪有关，当zoomType==ZoomType.geometric时cutType和cutGravity无效;
  String _assembly(
      {required String url,
      required double width,
      double height = 0,
      ImageCut cut = ImageCut.center,
      double radius = 0,
      ZoomType zoomType = ZoomType.geometric}) {
    var imageUrl = _imageUrl(url);
    return _aliyunZoomHandle(imageUrl, width, height, cut, radius, zoomType);
  }

  String assembly(
      {required String url,
      required double width,
      double height = 0,
      ImageCut cut = ImageCut.center,
      double radius = 0,
      ZoomType zoomType = ZoomType.geometric}) {
    return _assembly(url: url, width: width, height: height, cut: cut, radius: radius, zoomType: zoomType);
  }

  // Future<String> assemblyWith({
  //   required String url,
  //   required double width,
  //   double height = 0,
  //   ImageCut cut = ImageCut.center,
  //   double radius = 0,
  //   ZoomType zoomType = ZoomType.geometric,
  // }) async {
  //   var imageUrl = _imageUrl(url);
  //   var infoUrl = assemblyInfo(imageUrl);
  //   Retrofit retrofit = Retrofit(api: infoUrl, method: Method.GET);
  //   var imageInfo = await DioManager.instance.asyncRequestString(retrofit: retrofit);
  //   return _aliyunZoomHandle(imageUrl, width, height, cut, radius, zoomType);
  // }

  /// 配置图片url
  /// png图片时圆角裁剪无效
  /// [url] 原图片地址
  /// [width] 图片宽度(必填项)
  /// [height] 图片高度(裁剪时需要)
  /// [cutGravity] 图片裁剪对齐参考点(isPortrait==true时该参数无效),默认以center为参考点;
  /// [cutType] 裁剪方式，默认为圆角裁剪
  /// [radius] 裁剪半径、圆角半径;(默认为2，方便图片圆角半径参数设置)
  /// [zoomType] 缩放类型与裁剪有关，当zoomType==ZoomType.geometric时cutType和cutGravity无效;
  String assemblyTx({
    required String url,
    required double width,
    double height = 0,
    ImageCutGravity cutGravity = ImageCutGravity.center,
    ImageCutType cutType = ImageCutType.rradius,
    double radius = 2,
    ZoomType zoomType = ZoomType.geometric,
  }) {
    var type = _checkUrl(url);
    if (type == 2) {
      return url;
    } else if (type == 3) {
      var index = url.indexOf("?");
      url = url.substring(0, index);
    }
    if (url.suffixName == "gif") {
      return url;
    }
    var pixelRatio = MediaUtils.instance.pixelRatio;
    if (pixelRatio < 1) {
      pixelRatio = 1;
    } else {
      pixelRatio = pixelRatio * 1.2;
    }
    return _cutZoomAssembly(
        url: url,
        width: width * pixelRatio,
        height: height * pixelRatio,
        cutGravity: cutGravity,
        cutType: cutType,
        radius: radius,
        zoomType: zoomType);
  }

  /// 配置图片基本信息
  /// [url] 原图片地址
  String assemblyInfo(String url) {
    var type = _checkUrl(url);
    if (type == 2) {
      return url;
    } else if (type == 3) {
      var index = url.indexOf("?");
      url = url.substring(0, index);
    }
    if (url.suffixName == "gif") {
      return url;
    }
    var urlBuffer = StringBuffer(url);
    urlBuffer..write("?x-oss-process=image/info");
    return urlBuffer.toString();
  }

  /// 配置图片基本信息
  /// [url] 原图片地址
  String assemblyInfoTx(String url) {
    var type = _checkUrl(url);
    if (type == 2) {
      return url;
    } else if (type == 3) {
      var index = url.indexOf("?");
      url = url.substring(0, index);
    }
    if (url.suffixName == "gif") {
      return url;
    }
    var urlBuffer = StringBuffer(url);
    urlBuffer..write("?imageInfo");
    return urlBuffer.toString();
  }
}
