import 'package:eyflutter_core/device/permission_utils.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/mq/cloud_channel_manager.dart';
import 'package:eyflutter_uikit/uikit/pickers/beans/picker_model.dart';
import 'package:eyflutter_uikit/uikit/pickers/beans/pickers_arguments.dart';
import 'package:eyflutter_uikit/uikit/pickers/beans/pickers_langs.dart';
import 'package:eyflutter_uikit/uikit/pickers/beans/pickers_permission.dart';
import 'package:eyflutter_uikit/uikit/pickers/enums/pick_mode.dart';

/// 图片选择回调
typedef OnPickerSelectorCall = void Function(List<PickerModel> models);

/// 拍照回调
typedef OnPickerTakingCall = void Function(PickerModel model);

class PickersManager {
  factory PickersManager() => _getInstance();

  static PickersManager get instance => _getInstance();
  static PickersManager? _instance;

  static PickersManager _getInstance() {
    return _instance ??= new PickersManager._internal();
  }

  PickersManager._internal();

  String _showPickersMethodName = "hbQFcnLtYVZ9YrNHRqc4B2HRVY9yQ4Bz";

  PickersLangs? _langs;
  PickersArguments? _arguments;

  void _pickersSelector({
    PickMode mode = PickMode.all,
    required PickersPermission permission,
    required OnPickerSelectorCall selectorCall,
  }) {
    if ((_arguments?.maxCount ?? 0) <= 0) {
      _arguments?.maxCount = 1;
    }
    var languageJson = JsonUtils.toJson(_langs?.toJson() ?? {});
    var permissionJson = JsonUtils.toJson(permission.toJson());
    CloudChannelManager.instance.send<String>(_showPickersMethodName, arguments: {
      "pickerArguments": _arguments?.toJson() ?? null,
      "language": languageJson,
      "mode": mode.name,
      "permission": permissionJson
    }).then((response) => selectorCall(_pickersResult(response ?? "")), onError: (e) {
      //channel层通信异常有可能回调此方法
      selectorCall([]);
    });
  }

  List<PickerModel> _pickersResult(String response) {
    if (response.isEmptyString) {
      return [];
    }
    List<PickerModel> result = [];
    List _result = JsonUtils.fromJsons(response);
    _result.forEach((element) {
      PickerModel model = PickerModel(filePath: element["filePath"]);
      result.add(model);
    });
    return result;
  }

  /// 图片、视频选择器
  /// [arguments] 图片选择器参数
  /// [langArguments] 多语言配置参数
  /// [permissions] 审核拍照、图片、存储权限
  /// Android调用权限值：
  /// [AndroidPermission.readExternalStorage]
  /// [AndroidPermission.writeExternalStorage]
  /// [AndroidPermission.camera]
  /// iOS调用权限值：
  /// [mode] 选择模式 全部 照片 视频
  void showPickers({
    required PickersArguments arguments,
    required PickersLangs langArguments,
    required PickersPermission permission,
    required OnPickerSelectorCall selectorCall,
    PickMode mode = PickMode.all,
  }) {
    this._langs = langArguments;
    this._arguments = arguments;
    var prompt = _getPickerPrompt(permission: permission, mode: mode);
    var permissions = _getPickerPermissions(permission: permission, mode: mode);

    /// 这里的权限申请只处理打开选择器页面权限,
    /// 选择器内部操作若需要需在native层再次申请
    PermissionUtils.instance.apply(
        prompt: prompt,
        permissions: permissions,
        permissionCall: (granted) {
          if (granted) {
            _pickersSelector(mode: mode, permission: permission, selectorCall: selectorCall);
          } else {
            selectorCall([]);
          }
        });
  }

  String _getPickerPrompt({required PickersPermission permission, PickMode mode = PickMode.all}) {
    var prompt = permission.picker?.prompt ?? "";
    if (mode == PickMode.talkingAll) {
      prompt = permission.camera?.prompt ?? "";
    } else if (mode == PickMode.talkingImage) {
      prompt = permission.talkingImage?.prompt ?? "";
    } else if (mode == PickMode.talkingVideo) {
      prompt = permission.talkingVideo?.prompt ?? "";
    }
    return prompt;
  }

  List<String> _getPickerPermissions({required PickersPermission permission, PickMode mode = PickMode.all}) {
    var permissions = permission.picker?.permissions ?? [];
    if (mode == PickMode.talkingAll) {
      permissions = permission.camera?.permissions ?? [];
    } else if (mode == PickMode.talkingImage) {
      permissions = permission.talkingImage?.permissions ?? [];
    } else if (mode == PickMode.talkingVideo) {
      permissions = permission.talkingVideo?.permissions ?? [];
    }
    return permissions;
  }
}
