import 'dart:io';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/ring/cs_loading_container.dart';
import 'package:eyflutter_uikit/uikit/ring/icon_ring_indicator.dart';
import 'package:eyflutter_uikit/uikit/webkit/web_view_callback_handles.dart';
import 'package:eyflutter_webview/eyflutter_webview.dart';
import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher.dart';

typedef void OnWebInitializedCallback(WebViewCallbackHandles callbackHandles);

///加载已经结束
typedef void OnWebLoadFinished(String url);

/// 页面改变监听
typedef void OnWebPageChanged(String url);

///渲染web title
typedef void OnTitleCall(String title);

/// 页面返回监听
/// [isWebBack] web页面逐级回退时是否回至顶级页面
/// [url] 当前渲染的地址
typedef OnWebBack = void Function(bool isWebBack, String url);

typedef OnOverloadUrl = NavigationDecision Function(String url);

/// scheme拦截回调
typedef OnSchemeUrl = void Function(String schemeUrl);

/// 页面加载完成(包括资源)
typedef OnWebLoadCompletedCall = void Function(bool isFirst);

/// user agent回调
typedef OnUserAgentCall = void Function(String userAgent);

/// 更新title回调
typedef OnUpdateTitleCall = void Function(String title);

class H5WebView extends StatefulWidget {
  H5WebView({
    Key? key,
    required this.url,
    this.isEnabledDebug = false,
    this.overloadingUrl,
    this.headers,
    this.javascriptChannels,
    this.withJavascript = true,
    this.userAgent,
    this.withZoom = false,
    this.withDisplayZoomControls = false,
    this.initialChild,
    this.initializedCallback,
    this.backListener,
    this.clearCache = false,
    this.clearCookies = false,
    this.schemeCall,
    this.onWebLoadFinished,
    this.onPageChanged,
    this.onTitleCall,
    this.loadCompletedCall,
    this.isLoading = true,
    this.loadingText,
    this.titleCall,
    this.isOnlyChromeUserAgent = false,
  }) : super(key: key);

  ///remote or local url
  final String url;

  ///是否启用web view调试状态
  final bool isEnabledDebug;

  ///url改变监听
  final OnOverloadUrl? overloadingUrl;

  final Map<String, String>? headers;

  final Set<JavascriptChannel>? javascriptChannels;

  ///是否启用js交互(默认启用)
  final bool withJavascript;

  ///用户代理
  final String? userAgent;

  ///是否启用缩放(默认禁用)
  final bool withZoom;

  ///是否显示缩放控件(默认禁止)
  final bool withDisplayZoomControls;

  ///页面渲染完成前初始化视图
  final Widget? initialChild;

  /// 页面初始化完成时回调
  final OnWebInitializedCallback? initializedCallback;

  /// 页面返回监听
  final OnWebBack? backListener;

  ///加载页面时清除缓存
  final bool clearCache;

  ///清除页面cookies
  final bool clearCookies;

  ///request scheme url or javascript call scheme
  final OnSchemeUrl? schemeCall;

  ///加载已结束
  final OnWebLoadFinished? onWebLoadFinished;

  /// 页面改变监听
  final OnWebPageChanged? onPageChanged;

  ///web title listener
  final OnTitleCall? onTitleCall;

  /// 页面加载完成(包括资源)
  final OnWebLoadCompletedCall? loadCompletedCall;

  /// 是否显示loading
  final bool isLoading;

  /// loading text
  final String? loadingText;

  /// 更新标题回调
  final OnUpdateTitleCall? titleCall;

  final bool isOnlyChromeUserAgent;

  @override
  _H5WebViewState createState() => _H5WebViewState();
}

class _H5WebViewState extends State<H5WebView> implements WebViewCallbackHandles {
  ///前一次渲染url
  String _preUrl = "";
  bool isLoadingOffstage = true;
  WebViewController? controller;
  String currUrl = "";
  bool isCompletedFirstLoad = false;
  bool _isLoading = false;

  @override
  void initState() {
    super.initState();
    currUrl = widget.url;
    if (Platform.isAndroid) {
      WebView.platform = SurfaceAndroidWebView();
    }
    isCompletedFirstLoad = false;
    _isLoading = (widget.loadingText.isEmptyString ? false : widget.isLoading);
  }

  @override
  void dispose() {
    super.dispose();
    _preUrl = "";
    isCompletedFirstLoad = false;
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        WebView(
          key: widget.key,
          initialUrl: widget.url,
          javascriptMode: widget.withJavascript ? JavascriptMode.unrestricted : JavascriptMode.disabled,
          javascriptChannels: widget.javascriptChannels,
          debuggingEnabled: widget.isEnabledDebug,
          userAgent: widget.userAgent,
          onWebViewCreated: _onWebViewCreated,
          navigationDelegate: _navigationDelegate,
          onPageFinished: _onPageFinished,
          onPageChanged: widget.onPageChanged,
          onProgress: _onProgressCall,
          isOnlyChromeUserAgent: widget.isOnlyChromeUserAgent,
        ),
        _isLoading ? _buildLoading() : Stack(),
      ],
    );
  }

  Widget _buildLoading() {
    return CsLoadingContainer(
      text: widget.loadingText ?? "",
      indicator: IconRingIndicator(
        size: 20,
        indicatorBackgroundColor: Color(0xffb5bdc8),
        indicatorColor: Color(0xffe2e5e9),
        loadingType: LoadingType.classic,
      ),
    );
  }

  void _onProgressCall(int progress) {
    if (_isLoading && progress == 100) {
      setState(() {
        _isLoading = false;
      });
    }
    if (widget.loadCompletedCall != null && progress == 100) {
      isCompletedFirstLoad = true;
      if (isCompletedFirstLoad) {
        widget.loadCompletedCall!(false);
      } else {
        widget.loadCompletedCall!(true);
      }
    }
  }

  void _onPageFinished(String url) async {
    if (widget.onWebLoadFinished != null && _preUrl != url) {
      _preUrl = url;
      widget.onWebLoadFinished!(url);
    }
  }

  NavigationDecision _navigationDelegate(NavigationRequest request) {
    String url = request.url;
    if (url.isContainsSub(["javascript:", "about:blank"])) {
      return NavigationDecision.prevent;
    }
    if (url.isContainsSub(["smsto:", "tel:"])) {
      //native<->flutter权限检测
      _launchScheme(url);
      return NavigationDecision.prevent;
    }
    //若非http或https的url均作为scheme协议回调
    if (!url.isContainsStartsWith(["http", "https"])) {
      if (widget.schemeCall != null) {
        widget.schemeCall!(url);
      }
      return NavigationDecision.prevent;
    }
    NavigationDecision? decision;
    if (!url.isEmptyString) {
      currUrl = url;
      if (widget.overloadingUrl != null) {
        decision = widget.overloadingUrl!(url);
      }
    }
    if (decision == null || decision == NavigationDecision.prevent) {
      return NavigationDecision.prevent;
    }
    return NavigationDecision.navigate;
  }

  void _launchScheme(String scheme) async {
    if (await canLaunchUrl(Uri.parse(scheme))) {
      await launchUrl(Uri.parse(scheme));
    }
  }

  void _onWebViewCreated(WebViewController controller) async {
    this.controller = controller;
    if (widget.initializedCallback != null) {
      widget.initializedCallback!(this);
    }
    _titleCall();
    //清除缓存
    if (widget.clearCache) {
      controller.clearCache();
    }
    //清除cookies
    if (widget.clearCookies) {
      CookieManager().clearCookies();
    }
  }

  void _titleCall() async {
    if (widget.onTitleCall != null) {
      String title = (await controller?.getTitle()) ?? "";
      String patten = r"^((http|https|file|rtsp|mms)://[/]?)?(.*(/|.html).*)$";
      var regExp = new RegExp(patten, caseSensitive: false);
      var hasMatch = regExp.hasMatch(title);
      widget.onTitleCall!(hasMatch ? "" : title);
    }
  }

  @override
  Future<bool> canBack() {
    return controller?.canGoBack() ?? Future.value(false);
  }

  @override
  Future<String> currentUrl() async {
    var currentUrl = (await controller?.currentUrl()) ?? "";
    return Future.value(currentUrl);
  }

  @override
  void evalJavascript(String methodNameOrScript, List<Object> params, {bool isInjectionMethod = true}) {
    if (methodNameOrScript.isEmptyString || controller == null) {
      return;
    }
    if (isInjectionMethod) {
      StringBuffer bridge = new StringBuffer(methodNameOrScript);
      bridge.write("(");
      if (params.isEmptyList) {
        bridge.write(")");
      } else {
        for (int i = 0; i < params.length; i++) {
          Object value = params.elementAt(i);
          if ((value is int)) {
            bridge..write(value);
          } else {
            bridge
              ..write("'")
              ..write(value)
              ..write("'");
          }
          if ((i + 1) == params.length) {
            bridge..write(")");
          } else {
            bridge..write(",");
          }
        }
      }
      controller?.evaluateJavascript(bridge.toString());
    } else {
      controller?.evaluateJavascript(methodNameOrScript);
    }
  }

  @override
  void loadUrl(String url, {Map<String, String>? headers}) {
    if (url.isEmptyString) {
      return;
    }
    controller?.loadUrl(url, headers: headers);
  }

  @override
  Future<bool> onBack() async {
    var url = (await controller?.currentUrl()) ?? "";
    var canBackState = (await controller?.canGoBack()) ?? false;
    if (canBackState) {
      controller?.goBack();
    }
    currUrl = url;
    if (widget.backListener != null) {
      widget.backListener!(!canBackState, url);
    }
    canBackState = (await controller?.canGoBack()) ?? false;
    return Future.value(canBackState);
  }

  @override
  void reload() {
    controller?.reload();
    _titleCall();
  }

  @override
  void finish(BuildContext context, String pageName) {
    var navState = Navigator.of(context);
    if (navState.canPop()) {
      navState.pop(1);
    } else {
      RouteUtils.instance.finish(pageName);
    }
  }

  @override
  void clearCache() {
    controller?.clearCache();
  }

  @override
  void updateTitle(String title) {
    if (widget.titleCall != null) {
      widget.titleCall!(title);
    }
  }
}
