import 'package:eyflutter_uikit/uikit/widgets/check_button.dart';
import 'package:flutter/material.dart';

/// 排列方式
enum RadioArrange {
  /// 横向排列
  horizontal,

  /// 纵向排列
  vertical
}

/// radio对齐方式
enum AlignStyle {
  /// 靠左对齐
  wrap,

  /// 自动填充容器,仅在RadioArrange.horizontal时才生效
  autoFill
}

class RadioEntry {
  /// radio文本
  final String text;

  /// radio tag
  final String? tag;

  RadioEntry({required this.text, this.tag});
}

typedef OnRadioChangeCall = Function(int position, String tag);

class RadioGroup extends StatefulWidget {
  /// 排列方向
  final RadioArrange orientation;

  /// radio check icon
  final IconData checkIcon;

  /// radio uncheck icon
  final IconData uncheckIcon;

  /// 图标大小
  final double iconSize;

  /// 选择图标颜色
  final Color selectIconColor;

  /// 未选择图标颜色
  final Color unSelectIconColor;

  /// radio list
  final List<RadioEntry> items;

  /// 文本内边距
  final EdgeInsetsGeometry textPadding;

  /// 文本颜色
  final Color textColor;

  /// 文本大小
  final double textSize;

  /// 选项内边距
  final EdgeInsetsGeometry itemPadding;

  /// radio选择事件回调
  final OnRadioChangeCall? changeCall;

  /// 对齐方式
  final AlignStyle style;

  /// RadioGroup组件宽度
  /// 当style=AlignStyle.autoFill时该值必填
  final double? width;

  /// 默认被选中项tag
  final String? tag;

  /// 是否允许反选
  final bool allowReverseSelection;

  /// 是否填充容器
  final bool isTile;

  const RadioGroup({
    Key? key,
    required this.items,
    this.orientation = RadioArrange.horizontal,
    this.checkIcon = Icons.radio_button_on,
    this.uncheckIcon = Icons.radio_button_off,
    this.selectIconColor = const Color(0xffEEEBD7),
    this.unSelectIconColor = const Color(0xffEEEBD7),
    this.iconSize = 20,
    this.textColor = const Color(0xff333333),
    this.textSize = 12,
    this.textPadding = const EdgeInsets.only(left: 6),
    this.itemPadding = EdgeInsets.zero,
    this.changeCall,
    this.style = AlignStyle.wrap,
    this.width,
    this.tag,
    this.allowReverseSelection = true,
    this.isTile = false,
  }) : super(key: key);

  @override
  _RadioGroupState createState() => _RadioGroupState();
}

class _RadioGroupState extends State<RadioGroup> {
  Map<int, GlobalKey<CheckButtonState>> stateMap = {};

  @override
  Widget build(BuildContext context) {
    if (widget.orientation == RadioArrange.horizontal) {
      if (widget.style == AlignStyle.autoFill) {
        return SizedBox(
          width: widget.width,
          child: Table(
            children: [TableRow(children: buildItems())],
          ),
        );
      } else {
        return Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: buildItems(),
        );
      }
    } else {
      return Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: buildItems(),
      );
    }
  }

  List<Widget> buildItems() {
    List<Widget> lst = [];
    int position = 0;
    widget.items.forEach((element) {
      stateMap[position] = GlobalKey<CheckButtonState>();
      lst.add(Padding(
        padding: widget.itemPadding,
        child: CheckButton(
          key: stateMap[position],
          tag: element.tag,
          text: element.text,
          position: position,
          selectIconColor: widget.selectIconColor,
          unSelectIconColor: widget.unSelectIconColor,
          checkIcon: widget.checkIcon,
          uncheckIcon: widget.uncheckIcon,
          iconSize: widget.iconSize,
          textColor: widget.textColor,
          textSize: widget.textSize,
          textPadding: widget.textPadding,
          check: widget.tag == element.tag,
          allowReverseSelection: widget.allowReverseSelection,
          isTile: widget.isTile,
          checkedCall: (bool check, dynamic tag, int position) => onItemChange(check, position, element.tag ?? ""),
        ),
      ));
      position++;
    });
    return lst;
  }

  void onItemChange(bool check, int position, String tag) {
    stateMap.forEach((index, state) {
      if (position == index) {
        if (widget.allowReverseSelection) {
          state.currentState?.updateCheck(check);
        } else {
          state.currentState?.updateCheck(true);
        }
      } else {
        state.currentState?.updateCheck(false);
      }
    });
    if (widget.changeCall != null) {
      widget.changeCall!(position, tag);
    }
  }
}
