import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/options/item_option.dart';
import 'package:eyflutter_uikit/uikit/options/pop_widget.dart';
import 'package:eyflutter_uikit/uikit/state/lifecycle_state.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef OnFieldsControllerCall = void Function(OnFormFieldController controller);
typedef OnTextMenuItemClick = void Function(ItemOption itemOption);
typedef OnFieldCall = void Function(OnTextFieldController controller);

/// https://blog.csdn.net/ruoshui_t/article/details/90578161
mixin OnTextFieldListener {
  void onTextEditing(String tag, String text);

  void onTextEditComplete(String tag, String text);

  void onFocusListener(String tag, bool hasFocus) {}
}

mixin OnFormFieldController {
  /// true-验证通过,false-表单中至少有一项验证失败;
  bool validation();

  /// 更新提示信息
  /// [tag] TextField标识
  /// [tip] 提示信息
  void updateTipInfo(String tag, String tip);

  /// true-验证通过,false-表单中至少有一项验证失败;
  /// 只对状态判断不做提示信息更新
  bool validationState();

  /// 显示对应的文本菜单
  /// [tag] 文本标识
  void showTextMenu(String tag);

  /// 隐藏对应的文本菜单
  /// [tag] 文本标识
  void hideTextMenu(String tag);

  /// 更新文本值
  /// [tag] 文本标识
  /// [value] 文本值
  void updateText(String tag, String value);

  /// 获取文本
  /// [tag] 文本标识
  String getText(String tag);

  /// 更新密码可视
  /// [tag] 文本标识
  /// [value] 值
  void updateSecurity(String tag, bool value);
}

mixin OnTextFieldController {
  /// 清空文本
  void clear();

  /// 验证
  bool validation();

  /// 更新提示信息
  /// [tip] 提示信息
  void updateTipInfo(String tip);

  /// 更新文本
  void updateText(String text);

  /// 获取文本
  String getText();

  /// true-验证通过,false-表单中至少有一项验证失败;
  /// 只对状态判断不做提示信息更新
  bool validationState();

  //  更新密码可视
  void updateSecurity(bool value);
}

class TextFieldItem {
  /// 提示文本
  final String? hintText;

  /// 提示文本
  final TextStyle hintStyle;

  /// 输入文本样式
  final TextStyle labelStyle;

  /// 前缀文本
  final String? prefix;

  /// 前缀文本样式
  final TextStyle? prefixStyle;

  /// 输入框下面的提示信息样式
  final TextStyle tipTextStyle;

  /// 右下角显示文本
  final String? counterText;

  /// 右下角文本样式
  final TextStyle? counterStyle;

  /// 输入文本前组件
  final Widget? prefixWidget;

  /// 输入文本后组件
  Widget? suffixWidget;

  /// 下划线宽度
  final double underlineWidth;

  /// 下划线颜色
  final Color underlineColor;

  /// 错误下划线颜色
  final Color errorUnderlineColor;

  /// 选项标识
  final String? tag;

  /// 字段验证对象,回调参数为当前输入值,验证不通过时返回具体错误信息,通过时返回null;
  final FormFieldValidator<String>? validator;

  /// 最大宽度
  double? maxWidth;

  /// 最大高度
  double? maxHeight;

  /// 组件高度
  double? height;

  /// 启用密码
  bool enablePass;

  /// 输入限制
  List<TextInputFormatter>? inputFormatters;

  /// 键盘输入类型
  TextInputType? keyboardType;

  /// 菜单选项
  List<ItemOption> menuItems;

  /// 默认文本
  String? defaultText;

  /// 菜单偏移量
  Offset? menuOffset;

  /// 最小行号
  int minLines;

  /// 最大行号
  int maxLines;

  /// 内容对齐
  TextAlign textAlign;

  /// 是否隐藏组件
  bool visible;

  /// 边框装饰
  final BoxBorder? border;

  /// 边框圆角
  final BorderRadiusGeometry? borderRadius;

  /// 内边距
  final EdgeInsetsGeometry? contentPadding;

  TextFieldItem({
    this.hintText,
    this.hintStyle = const TextStyle(color: Color(0xff999999), fontSize: 12),
    this.labelStyle = const TextStyle(color: Color(0xff333333), fontSize: 13),
    this.prefix,
    this.prefixStyle,
    this.tipTextStyle = const TextStyle(color: Color(0x99ff2e18), fontSize: 12),
    this.counterText,
    this.counterStyle,
    this.prefixWidget,
    this.suffixWidget,
    this.underlineWidth = 1,
    this.underlineColor = const Color(0xffEFEFEF),
    this.errorUnderlineColor = const Color(0xffEFEFEF),
    this.tag,
    this.validator,
    this.maxWidth,
    this.maxHeight,
    this.height,
    this.enablePass = false,
    this.inputFormatters,
    this.keyboardType,
    this.menuItems = const [],
    this.defaultText,
    this.menuOffset,
    this.minLines = 1,
    this.maxLines = 1,
    this.textAlign = TextAlign.left,
    this.visible = true,
    this.border,
    this.borderRadius,
    this.contentPadding,
  });
}

class TextFieldWidget extends StatefulWidget {
  final TextFieldItem? fieldItem;
  final OnTextFieldListener? fieldListener;
  final OnFieldCall? fieldCall;

  /// 输入框外边距
  final EdgeInsetsGeometry? margin;

  /// 默认文本
  final String? defaultText;

  /// 文本菜单选项宽度
  final double? textMenuItemWidth;

  /// 文本菜单选项高度
  final double? textMenuItemHeight;

  /// 文本菜单选项单击事件
  final OnTextMenuItemClick? menuItemClick;

  /// 菜单图标颜色
  final Color? menuIconColor;

  const TextFieldWidget(
      {Key? key,
      this.fieldItem,
      this.fieldListener,
      this.fieldCall,
      this.margin,
      this.defaultText,
      this.textMenuItemWidth,
      this.textMenuItemHeight,
      this.menuItemClick,
      this.menuIconColor})
      : super(key: key);

  @override
  _TextFieldWidgetState createState() => _TextFieldWidgetState();
}

class _TextFieldWidgetState extends State<TextFieldWidget> with OnTextFieldController {
  FocusNode focusNode = new FocusNode(descendantsAreFocusable: false);
  late TextEditingController _editingController;
  String _labelText = "";
  String? _tipText;
  int prevTime = 0;
  bool _isShowMenu = false;
  String _currentText = "";

  @override
  void initState() {
    super.initState();
    _currentText = widget.defaultText ?? "";
    _editingController = TextEditingController(text: _currentText);
    focusNode.addListener(() {
      setState(() {
        _bindHintText();
      });
      widget.fieldListener?.onFocusListener(widget.fieldItem?.tag ?? "", focusNode.hasFocus);
    });
    _bindHintText();
    _editingController.addListener(() {
      _currentText = _editingController.text;
      _inputValid();
      widget.fieldListener?.onTextEditing(widget.fieldItem?.tag ?? "", _currentText);
    });
    if (widget.fieldCall != null) {
      widget.fieldCall!(this);
    }
  }

  void _bindHintText() {
    if (focusNode.hasFocus) {
      _labelText = "";
    } else {
      _labelText = widget.fieldItem?.hintText ?? "";
    }
  }

  @override
  void dispose() {
    super.dispose();
    focusNode.unfocus();
    _editingController.dispose();
  }

  int currentTimeMillis() {
    return new DateTime.now().millisecondsSinceEpoch;
  }

  void _inputValid() {
    int currTime = currentTimeMillis();
    int diff = currTime - prevTime;
    if (diff > 700) {
      prevTime = currTime;
      validation();
    }
  }

  UnderlineInputBorder underlineBorder(Color color) {
    return UnderlineInputBorder(
      borderSide:
          BorderSide(color: color, width: (widget.fieldItem?.underlineWidth ?? 0.9) * 0.6, style: BorderStyle.solid),
      borderRadius: BorderRadius.all(Radius.circular(2)),
    );
  }

  Widget _buildTextMenu(BuildContext context) {
    return PopupMenuButton<String>(
      icon: Icon(
        _isShowMenu ? Icons.arrow_drop_up : Icons.arrow_drop_down,
        color: widget.menuIconColor,
      ),
      iconSize: 20,
      offset: widget.fieldItem?.menuOffset ?? Offset(0, 52),
      onSelected: (String value) => _menuClick(value),
      onCanceled: () {
        setState(() {
          _isShowMenu = false;
        });
      },
      padding: EdgeInsets.zero,
      itemBuilder: (BuildContext context) {
        setState(() {
          _isShowMenu = true;
        });
        return widget.fieldItem?.menuItems
                .map((e) => PopupMenuItem<String>(
                      child: Container(
                        width: 400,
                        height: widget.textMenuItemHeight ?? 36,
                        alignment: Alignment.centerLeft,
                        child: Text(e.title ?? ""),
                      ),
                      value: e.title,
                      height: widget.textMenuItemHeight ?? 36,
                      textStyle: TextStyle(fontSize: 14, color: Color(0xff262626)),
                    ))
                .toList() ??
            [];
      },
    );
  }

  void _menuClick(String value) {
    setState(() {
      _isShowMenu = false;
    });
    updateText(value);
    if (widget.menuItemClick != null) {
      widget.fieldItem?.menuItems.forEach((element) {
        if (value == "${element.title}") {
          widget.menuItemClick!(element);
          return;
        }
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Visibility(
      child: Container(
        height: widget.fieldItem?.height,
        constraints: BoxConstraints(
            maxWidth: widget.fieldItem?.maxWidth ?? double.infinity,
            maxHeight: widget.fieldItem?.maxHeight ?? double.infinity),
        margin: widget.margin ?? EdgeInsets.zero,
        decoration: BoxDecoration(
          border: widget.fieldItem?.border,
          borderRadius: widget.fieldItem?.borderRadius,
        ),
        alignment: Alignment.topLeft,
        child: TextField(
          style: widget.fieldItem?.labelStyle,
          onEditingComplete: () {
            if (_editingController.value.text.isEmptyString) {
              _inputValid();
              widget.fieldListener?.onTextEditComplete(widget.fieldItem?.tag ?? "", _editingController.value.text);
              focusNode.unfocus();
            }
            SystemChannels.textInput.invokeMethod('TextInput.hide');
          },
          inputFormatters: widget.fieldItem?.inputFormatters,
          controller: _editingController,
          enabled: true,
          focusNode: focusNode,
          cursorWidth: 1,
          cursorHeight: 20,
          textAlign: widget.fieldItem?.textAlign ?? TextAlign.left,
          cursorColor: Color(0xcc222222),
          obscureText: widget.fieldItem?.enablePass ?? false,
          keyboardType: widget.fieldItem?.keyboardType ?? TextInputType.multiline,
          textInputAction: TextInputAction.done,
          minLines: widget.fieldItem?.minLines,
          maxLines: widget.fieldItem?.maxLines,
          decoration: InputDecoration(
            //并且没有输入文字时提示
            hintText: _labelText,
            //设置输入文本框的提示文字的样式
            hintStyle: widget.fieldItem?.hintStyle,
            //输入框没有获取焦点时显示
            labelText: isEmptyText() ? _labelText : "",
            labelStyle: widget.fieldItem?.hintStyle,
            //前缀文本
            prefixText: widget.fieldItem?.prefix,
            prefixStyle: widget.fieldItem?.prefixStyle,
            //错误信息
            errorText: _tipText,
            errorStyle: widget.fieldItem?.tipTextStyle,
            //输入框右下角显示的文本
            counterText: widget.fieldItem?.counterText,
            counterStyle: widget.fieldItem?.counterStyle,
            //文字前的小图标
            prefix: widget.fieldItem?.prefixWidget,
            //文字后面的小图标
            suffixIcon: _buildSuffixWidget(context),
            border: underlineBorder(widget.fieldItem?.underlineColor ?? Colors.transparent),
            enabledBorder: underlineBorder(widget.fieldItem?.underlineColor ?? Colors.transparent),
            errorBorder: underlineBorder(widget.fieldItem?.errorUnderlineColor ?? Colors.transparent),
            focusedErrorBorder: underlineBorder(widget.fieldItem?.underlineColor ?? Colors.transparent),
            disabledBorder: underlineBorder(widget.fieldItem?.underlineColor ?? Colors.transparent),
            focusedBorder: underlineBorder(widget.fieldItem?.errorUnderlineColor ?? Colors.transparent),
            contentPadding: widget.fieldItem?.contentPadding,
          ),
        ),
      ),
      visible: widget.fieldItem?.visible ?? true,
    );
  }

  bool isEmptyText() {
    if (_currentText.isEmptyString && _editingController.value.text.isEmptyString) {
      return true;
    } else {
      return false;
    }
  }

  Widget? _buildSuffixWidget(BuildContext context) {
    if (widget.fieldItem?.menuItems.isNotEmptyList ?? false) {
      return _buildTextMenu(context);
    }
    return widget.fieldItem?.suffixWidget;
  }

  @override
  void clear() {
    _editingController.clear();
  }

  @override
  bool validation() {
    if (widget.fieldItem?.validator == null) {
      return false;
    }
    var validTipText = widget.fieldItem?.validator!(_editingController.text);
    var pass = validTipText.isEmptyString;
    setState(() {
      if (pass) {
        _tipText = null;
      } else {
        _tipText = validTipText;
      }
    });
    return pass;
  }

  @override
  void updateTipInfo(String tip) {
    setState(() {
      if (tip.isEmptyString) {
        _tipText = null;
      } else {
        _tipText = tip;
      }
    });
  }

  @override
  void updateSecurity(bool value) {
    setState(() {});
  }

  @override
  bool validationState() {
    if (widget.fieldItem?.validator == null) {
      return false;
    }
    var validTipText = widget.fieldItem?.validator!(_editingController.text);
    var pass = validTipText.isEmptyString;
    return pass;
  }

  @override
  void updateText(String text) {
    _currentText = text;
    _editingController.value = _editingController.value.copyWith(
      text: text,
      selection: TextSelection(baseOffset: text.length, extentOffset: text.length),
      composing: TextRange.empty,
    );
    focusNode.unfocus();
    setState(() {});
  }

  @override
  String getText() {
    return _editingController.value.text;
  }
}

class TextFields extends StatefulWidget {
  final List<TextFieldItem>? items;
  final OnTextFieldListener? fieldListener;
  final OnFieldsControllerCall? controllerCall;

  /// 最大宽度
  final double? maxWidth;

  /// 最大高度
  final double? maxHeight;

  /// 输入框外边距
  final EdgeInsetsGeometry? itemMargin;

  /// 文本菜单选项高度
  final double textMenuItemHeight;

  /// 文本菜单选项单击事件
  final OnTextMenuItemClick? menuItemClick;

  /// 菜单图标颜色
  final Color menuIconColor;

  /// 背景颜色
  final Color? backgroundColor;

  /// 边框圆角
  final BorderRadiusGeometry? borderRadius;

  /// 组件内边距
  final EdgeInsetsGeometry? padding;

  /// 组件外边距
  final EdgeInsetsGeometry? margin;

  const TextFields({
    Key? key,
    this.items,
    this.fieldListener,
    this.maxWidth,
    this.maxHeight,
    this.controllerCall,
    this.itemMargin,
    this.textMenuItemHeight = 36,
    this.menuItemClick,
    this.menuIconColor = const Color(0xff262626),
    this.backgroundColor,
    this.borderRadius,
    this.padding,
    this.margin,
  }) : super(key: key);

  @override
  _TextFieldsState createState() => _TextFieldsState();
}

class _TextFieldsState extends LifecycleState<TextFields> with OnFormFieldController {
  Map<String, OnTextFieldController> _fieldControllers = {};
  Map<String, PopWidgetController> popControllers = {};

  @override
  String get viewName => "textFields";

  @override
  void onCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    if (widget.controllerCall != null) {
      widget.controllerCall!(this);
    }
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return Container(
      padding: widget.padding,
      margin: widget.margin,
      decoration: BoxDecoration(
        color: widget.backgroundColor,
        borderRadius: widget.borderRadius,
      ),
      child: Column(
        children: _buildFields(),
      ),
    );
  }

  List<Widget> _buildFields() {
    List<Widget> lst = [];
    widget.items?.forEach((element) {
      if (element.maxWidth == null) {
        element.maxWidth = widget.maxWidth;
      }
      if (element.maxHeight.isNullOrZero) {
        element.maxHeight = widget.maxHeight;
      }
      lst.add(_buildTextField(element));
    });
    return lst;
  }

  Widget _buildTextField(TextFieldItem fieldItem) {
    return TextFieldWidget(
      fieldItem: fieldItem,
      fieldListener: widget.fieldListener,
      fieldCall: (OnTextFieldController controller) {
        if (fieldItem.tag.isNotEmptyString) {
          _fieldControllers[fieldItem.tag!] = controller;
        }
      },
      margin: widget.itemMargin,
      defaultText: fieldItem.defaultText,
      textMenuItemHeight: widget.textMenuItemHeight,
      menuItemClick: widget.menuItemClick,
      menuIconColor: widget.menuIconColor,
    );
  }

  @override
  bool validation() {
    bool status = true;
    _fieldControllers.forEach((key, element) {
      if (!element.validation()) {
        status = false;
        return;
      }
    });
    return status;
  }

  @override
  void updateSecurity(String tag, bool value) {
    var controller = _fieldControllers[tag];
    controller?.updateSecurity(value);
  }

  @override
  void updateTipInfo(String tag, String tip) {
    var controller = _fieldControllers[tag];
    controller?.updateTipInfo(tip);
  }

  @override
  bool validationState() {
    bool pass = true;
    _fieldControllers.forEach((key, controller) {
      if (!controller.validationState()) {
        pass = false;
        return;
      }
    });
    return pass;
  }

  @override
  void showTextMenu(String tag) {
    var controller = popControllers[tag];
    if (controller == null) {
      return;
    }
    controller.showMenu();
  }

  @override
  void hideTextMenu(String tag) {
    var controller = popControllers[tag];
    if (controller == null) {
      return;
    }
    controller.hideMenu();
  }

  @override
  void updateText(String tag, String value) {
    var controller = _fieldControllers[tag];
    controller?.updateText(value);
  }

  @override
  String getText(String tag) {
    var controller = _fieldControllers[tag];
    return controller?.getText() ?? "";
  }
}
